login_query="""{
  viewer{
    login
  }
}"""

all_query = """
{
  repository(owner: "%s", name: "%s") {
    name
    pullRequests(first:100){
      pageInfo{
        endCursor
        hasNextPage
      }
      nodes{
        publishedAt
        id
      }
    }
    ref(qualifiedName: "master"){
      target{
        ... on Commit{
          history(first:100){
            pageInfo {
              endCursor
              hasNextPage
            }
            totalCount
            edges {
              node {
                author{
                  email
                }
                pushedDate
              }
            }
          }
        }
      }
    }
  }
}
"""
all_query_with_pager = """
{
  repository(owner: "%s", name: "%s") {
    name
    pullRequests(first:100,after:"%s"){
      pageInfo{
        endCursor
        hasNextPage
      }
      nodes{
        publishedAt
        id
      }
    }
    ref(qualifiedName: "master"){
      target{
        ... on Commit{
          history(first:100,after:"%s"){
            pageInfo {
              endCursor
              hasNextPage
            }
            totalCount
            edges {
              node {
                author{
                  email
                }
                pushedDate
              }
            }
          }
        }
      }
    }
  }
}
"""

pr_query_with_pager ="""
{
  repository(owner: "%s", name: "%s") {
    name
    pullRequests(first:100,after:"%s"){
      pageInfo{
        endCursor
        hasNextPage
      }
      nodes{
        publishedAt
        id
      }
    }
  }
}
"""

commit_query_with_pager = """
{
  repository(owner: "%s", name: "%s") {
    name
    ref(qualifiedName: "master"){
      target{
        ... on Commit{
          history(first:100,after:"%s"){
            pageInfo {
              endCursor
              hasNextPage
            }
            totalCount
            edges {
              node {
                author{
                  email
                }
                pushedDate
              }
            }
          }
        }
      }
    }
  }
}
"""

organ_all_query="""
{
  organization(login:"%s"){
    repositories(first:100,orderBy:{field:STARGAZERS,direction:DESC}){
      pageInfo{
        hasNextPage
        endCursor
      }
      nodes{
        name,
        stargazers{
          totalCount
        },
      }
    }
  }
}
"""

organ_all_query_with_pager="""
{
  organization(login:"%s"){
    repositories(first:100,after:"%s",orderBy:{field:STARGAZERS,direction:DESC}){
      pageInfo{
        hasNextPage
        endCursor
      }
      nodes{
        name,
        stargazers{
          totalCount
        },
      }
    }
  }
}
"""

user_all_query="""
{
  user(login:"%s"){
    repositories(first:100,orderBy:{field:STARGAZERS,direction:DESC}){
      pageInfo{
        hasNextPage
        endCursor
      }
      nodes{
        name,
        owner {
          login
        }
        stargazers{
          totalCount
        },
      }
    }
  }
}
"""

user_all_query_with_pager="""
{
  organization(login:"%s"){
    repositories(first:100,after:"%s",orderBy:{field:STARGAZERS,direction:DESC}){
      pageInfo{
        hasNextPage
        endCursor
      }
      nodes{
        name,
        owner {
          login
        }
        stargazers{
          totalCount
        },
      }
    }
  }
}
"""


from ..libs import query
from ..libs import helpers
import pandas as pd
import numpy as np
import click
import math
def analyse_repo(owner,repository,config):
    """查询 Repo 数据"""
    # 定义空白数组
    commitArray = []
    pullRequestArray = []

    # 添加一个可过滤掉的数据，确保后续执行完成
    commitArray.append({
            'author' : 'localhost',
            'date': '未标注时间',
            "times":1
        })
    pullRequestArray.append({
            'date': '未标注时间',
            "times":1
        })
    # 定义查询变量
    start_time=config["time"]["start_time"]
    end_time=config["time"]["end_time"]
    top_number=int(config["rank"]["top"])
    click.echo("初次抓取数据:%s/%s" % (owner,repository))
    # 进行初次查询
    all_query = query.all_query % (owner, repository)
    result = helpers.query(all_query,config)
    # 生成时间 Series
    date_range = pd.date_range(start=start_time,end=end_time,freq="W")
    date_series = pd.Series(np.zeros((len(date_range),), dtype=int),index=date_range)

    # 处理第一组数据
    if (helpers.has_result(result,"commit")):
        for commit in result["data"]["repository"]["ref"]["target"]["history"]["edges"]:
            helpers.add_item_to_commit_array(commit,commitArray)
            pass

    if (helpers.has_result(result,"pr")):
        for pullRequest in result["data"]["repository"]["pullRequests"]["nodes"]:
            helpers.add_item_to_pr_array(pullRequest,pullRequestArray)


    while helpers.has_next_page(result,"commit") or helpers.has_next_page(result,"issue") or helpers.has_next_page(result,"pr"):
        click.echo("继续抓取数据:%s/%s" % (owner,repository));
        if (helpers.has_result(result,"commit")):
            for commit in result["data"]["repository"]["ref"]["target"]["history"]["edges"]:
                helpers.add_item_to_commit_array(commit,commitArray)
                pass
        if (helpers.has_result(result,"pr")):
            for pullRequest in result["data"]["repository"]["pullRequests"]["nodes"]:
                helpers.add_item_to_pr_array(pullRequest,pullRequestArray)

        if (helpers.has_next_page(result,"pr") and helpers.has_next_page(result,"commit")):
            next_query = query.all_query_with_pager % (owner,repository,helpers.get_page_cursor(result,"pr"),helpers.get_page_cursor(result,"commit"))
        elif (helpers.has_next_page(result,"pr")):
            next_query = query.pr_query_with_pager % (owner,repository,helpers.get_page_cursor(result,"pr"))
        elif (helpers.has_next_page(result,"commit")):
            next_query = query.commit_query_with_pager % (owner,repository,helpers.get_page_cursor(result,"commit"))

        result = helpers.query(next_query,config)

    print("分析 PR")
    pr_frame = pd.DataFrame(pullRequestArray);
    if not pr_frame.empty:
        pr_frame = pr_frame[pr_frame.date != "未标注时间"]
        pr_frame["date"] = pd.to_datetime(pr_frame['date'])
        pr_dstList = pr_frame.set_index('date').resample('W')['times'].sum()
        pr_dstList = pr_dstList.loc[start_time:end_time]

    print("分析 commit")

    commit_frame = pd.DataFrame(commitArray);

    # commit_frame.to_pickle("output/commits.pkl")
    commit_frame = commit_frame[commit_frame.date != "未标注时间"]
    commit_frame["date"] = pd.to_datetime(commit_frame['date'])
    commit_dstList = commit_frame.set_index('date').resample('W')['times'].sum()
    commit_dstList = commit_dstList.loc[start_time:end_time]

    print("分析 Contributor")
    contributor_frame = pd.DataFrame(commitArray);

    contributor_frame = contributor_frame[contributor_frame.date != "未标注时间"]
    contributor_frame["date"] = pd.to_datetime(contributor_frame['date'])
    contributor_dstList = contributor_frame.drop_duplicates(subset=["author"]).set_index('date').resample('W')['times'].sum()
    contributor_dstList = contributor_dstList.loc[start_time:end_time]


    new_commit_series = pd.Series(np.zeros((len(date_range),), dtype=int),index=date_range)
    for item in commit_dstList.index:
        if item in date_series.index:
            new_commit_series[item] = commit_dstList[item]


    new_pr_series = pd.Series(np.zeros((len(date_range),), dtype=int),index=date_range)
    for item in pr_dstList.index:
        if item in date_series.index:
            new_pr_series[item] = pr_dstList[item]


    new_contributor_series = pd.Series(np.zeros((len(date_range),), dtype=int),index=date_range)
    for item in contributor_dstList.index:
        if item in date_series.index:
            new_contributor_series[item] = contributor_dstList[item]

        # 构成新的 DataFrame

    new_df = pd.DataFrame({
        "contributor":new_contributor_series.values,
        "commit":new_commit_series.values,
        "pr":new_pr_series.values
    },index = date_range)

    # 计算活跃分数

    new_df["score"] = new_df.apply(lambda row: math.sqrt(row.pr*row.pr + row.contributor * row.contributor + row.commit*row.commit), axis=1)

    # 求活跃分数平均值

    target_score = new_df["score"].sum() / len(new_df)

    # 获取平均分实例，用于后续排序

    instance = helpers.get_avarage_instance()

     # 将项目的活跃分数保存到新的 Pickle 中，用于后续的折线图输出

    helpers.series_to_pickle(new_df,repository)

    # 对平均分实例进行排序

    helpers.set_avarage(instance,owner,repository,target_score)

    # 输出项目的 CSV 数据
    helpers.export_csv(new_df,"%s" % repository)

    # 生成折线图

    helpers.generate_line_number(start_time,end_time,top_number)
    click.echo("输出成功,%s 旗下的 %s 项目的活跃分数为 %.15f"% (owner,repository,target_score))
    click.echo("排行榜及折线图请查看 result 目录")


def analyse_organ(organization,config):
    repositoryArray = []
    click.echo("开始抓取组织数据:%s" % organization)
    all_query = query.organ_all_query % organization
    result = helpers.query(all_query,config)
    if (helpers.has_result(result,'repository')):
            for repo in result["data"]["organization"]["repositories"]["nodes"]:
                if repo['stargazers']["totalCount"] > 100:
                    repositoryArray.append({"owner":organization,"repository":repo["name"]})

    while(helpers.has_next_page(result,'repository')):
        click.echo("继续抓取组织数据: %s" % organization)
        if (helpers.has_result(result,'repository')):
            for repo in result["data"]["organization"]["repositories"]["nodes"]:
                if repo['stargazers']["totalCount"] > 100:
                    repositoryArray.append({"owner":organization,"repository":repo["name"]})

        click.echo("继续抓取组织数据:%s" % organization)
        next_query = query.organ_all_query_with_pager % (organization,helpers.get_page_cursor(result,"repository"))
        result = helpers.query(next_query,config)

    for item in repositoryArray:
        analyse_repo(item["owner"],item["repository"],config)

def analyse_user(user,config):
    repositoryArray = []
    click.echo("开始抓取用户数据:%s" % user)
    all_query = query.user_all_query % user
    result = helpers.query(all_query,config)

    if (helpers.has_result(result,'user_repository')):
            for repo in result["data"]["user"]["repositories"]["nodes"]:
                if repo['stargazers']["totalCount"] > 100 and repo["owner"]["login"].lower() == user.lower():
                    repositoryArray.append({"owner":user,"repository":repo["name"]})

    while(helpers.has_next_page(result,'user_repository')):
        click.echo("继续抓取用户数据: %s" % user)
        if (helpers.has_result(result,'user_repository')):
            for repo in result["data"]["user"]["repositories"]["nodes"]:
                if repo['stargazers']["totalCount"] > 100:
                    repositoryArray.append({"owner":user,"repository":repo["name"]})

        click.echo("继续抓取用户数据:%s" % user)
        next_query = query.organ_all_query_with_pager % (organization,helpers.get_page_cursor(result,"user_repository"))
        result = helpers.query(next_query,config)

    for item in repositoryArray:
        analyse_repo(item["owner"],item["repository"],config)




import requests
import os
import configparser
import datetime
import click
import shutil
import pandas as pd
import matplotlib

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt

def query(query,config):
    """封装后的 GraphQL 请求"""
    if config["login"]["token"] == '':
        print("You Need Login First, Run `grank login`")
        exit()
    token = config["login"]["token"]
    headers = {"Authorization": "Bearer %s" % token}
    response = requests.post('https://api.github.com/graphql', json={'query': query}, headers=headers)
    if response.status_code == 200:
        return response.json()
    else:
        raise Exception("Query failed to run by returning code of {}. {}".format(response.status_code, query))

def check_exist():
    """检测所需文件是否存在"""
    if not os.path.exists('result'):
        os.makedirs('result')
    if not os.path.exists('output'):
        os.makedirs('output')
    if os.path.isfile('grank.ini'):
        return True
    else:
        return False

def get_config():
    """读取配置文件并返回对应实例"""
    configInstance = configparser.ConfigParser()
    if not check_exist():
        configInstance = configparser.ConfigParser()
        configInstance["login"] = {}
        configInstance["login"]["token"]= ''
        configInstance["corp"] = {}
        configInstance["corp"]["keyword"]=''
        configInstance["time"] = {}
        configInstance["time"]["start_time"]='2017-01-01'
        configInstance["time"]["end_time"]= datetime.date.today().strftime('%Y-%m-%d') # 使用今天的日期
        configInstance["rank"] = {}
        configInstance["rank"]["top"] = '3' # 默认制作前三名的综合图像
        with open('grank.ini', 'w') as configfile:
            configInstance.write(configfile)
    configInstance.read('grank.ini')
    return configInstance
    pass

def set_user_token(token):
    """向配置文件写入用户 Token"""
    config = get_config();
    config["login"]["token"] = token
    with open('grank.ini','w') as configfile:
        config.write(configfile)

def set_keyword(keyword):
    """向配置文件写入关键词"""
    config = get_config();
    config["corp"]["keyword"] = keyword
    with open('grank.ini','w') as configfile:
        config.write(configfile)

def has_next_page(result,mode):
    """判断是否有下一页"""
    if mode == "pr":
        if (has_result(result,"pr")):
            if result["data"]["repository"]["pullRequests"]["pageInfo"]["hasNextPage"]:
                return True
            else:
                return False
        else:
            return False

    if mode == "commit":
        if (has_result(result,"commit")):
            if result["data"]["repository"]["ref"]['target']["history"]["pageInfo"]["hasNextPage"]:
                return True
            else:
                return False
        else:
            return False
    if mode == "repository":
        if (has_result(result,"repository")):
            if result["data"]["organization"]["repositories"]["pageInfo"]["hasNextPage"]:
                return True
            else:
                return False
        else:
            return False

    if mode == "user_repository":
        if (has_result(result,"user_repository")):
            if result["data"]["user"]["repositories"]["pageInfo"]["hasNextPage"]:
                return True
            else:
                return False
        else:
            return False

def get_page_cursor(result,mode):
    """判断是否有对应的结果"""
    if mode == "pr":
        return result["data"]["repository"]["pullRequests"]["pageInfo"]["endCursor"]

    if mode == "commit":
        return result["data"]["repository"]["ref"]['target']["history"]["pageInfo"]["endCursor"]

    if mode == "repository":
        return result["data"]["organization"]["repositories"]["pageInfo"]["endCursor"]

    if mode == "user_repository":
        return result["data"]["organization"]["repositories"]["pageInfo"]["endCursor"]

def has_result(result,mode):
    """判断是否有对应的结果"""
    if mode == "pr":
        return ("pullRequests" in result["data"]["repository"])
    elif mode == "commit":
        return ("ref" in result["data"]["repository"])
    elif mode == 'repository':
        return ('repositories' in result["data"]["organization"])
    elif mode == 'user_repository':
        return ('repositories' in result["data"]["user"])

def cover_time(time):
    """时间的简化处理"""
    if time :
        return time[0:10]
    else:
        return "未标注时间"

def add_item_to_commit_array(item,blank_array):
    """针对 commit 的数组处理"""
    blank_array.append({
            'author' : item["node"]["author"]["email"],
            'date': cover_time(item["node"]["pushedDate"]),
            "times":1
        })

def add_item_to_pr_array(item,blank_array):
    """针对 pull requests 的数组处理"""
    blank_array.append({
      "date":cover_time(item["publishedAt"]),
      "times":1
    })


def export_csv(series,name):
    """导出 Csv 文件"""
    series.to_csv("output/%s.csv" % name);

def get_avarage_instance():
    """获取平均值 DF 实例"""
    if not os.path.isfile("output/average.pkl"):
        pd.DataFrame(data={'name': [], 'score': []}).to_pickle("output/average.pkl")
    return pd.read_pickle("output/average.pkl")

def set_avarage(instance,owner,repository,score):
    """保存中间值，并更新 csv 文件"""
    instance = instance.append(pd.Series({"owner":owner,"name":repository,"score":score}),ignore_index=True)
    instance = instance.drop_duplicates(subset=["owner","name"]).sort_values(["score"],ascending=False)
    instance.to_pickle("output/average.pkl")
    instance.to_csv("result/project_rank.csv")

def series_to_pickle(df,name):
    """将数据保存到 pickle 中"""
    df.to_pickle("output/%s.pkl" % name)

def generate_line_number(start_time,end_time,top_number):
    """生成平均值的折线图"""
    df = pd.read_pickle("output/average.pkl")
    all_df = pd.DataFrame(data=[],index=pd.date_range(start=start_time,end=end_time,freq="W"))

    for index, row in df.iterrows():
        if len(all_df.columns) < top_number:
            all_df[row["name"]] = pd.read_pickle("output/%s.pkl" % row["name"])["score"]
        else:
            break

    fig = all_df.plot().get_figure()
    fig.savefig("result/line.png")
    plt.close(fig)

def clean_directory():
    """清空临时目录及结果目录"""
    shutil.rmtree('output',ignore_errors=True)
    shutil.rmtree('result',ignore_errors=True)
    click.echo("Workspace is empty now!")
    pass


import click
import sys

if sys.version_info[0] != 3:
    """设置 Python 3 的判断如果不是 Python 3 则退出"""
    print("This script requires Python 3")
    exit()

from .libs import helpers
from .libs import query
from .script import activity

@click.group()
def main():
    """Grank Command"""
    pass

@main.command()
@click.option('--keyword', prompt=True,help="The Corp Keyword")
def config(keyword):
    """ Config Grank"""
    helpers.set_keyword(keyword)
    click.echo('Update Grank.ini Success!')
    pass

@main.command()
@click.option('--token', prompt='Your Personal Access Token',
              help='Generate at https://github.com/settings/tokens')
def login(token):
    """Login with Personal Access Token"""
    helpers.set_user_token(token)
    click.echo('Login Success!')
    pass


@main.command()
def checklogin():
    """ Check User Login"""
    config = helpers.get_config()
    result = helpers.query(query.login_query,config)
    click.echo('Your Username is %s' % result["data"]["viewer"]["login"]);
    pass

@main.command()
@click.argument('organization')
def organ(organization):
    """Analyse a Github Organization"""
    config = helpers.get_config()
    activity.analyse_organ(organization,config)
    pass

@main.command()
@click.argument('organization')
@click.argument('repo')
def repo(organization,repo):
    """Analyse a Github Repository"""
    config = helpers.get_config()
    activity.analyse_repo(organization,repo,config)
    pass

@main.command()
@click.argument('user')
def user(user):
    """Analyse a Github User"""
    config = helpers.get_config()
    activity.analyse_user(user,config)
    pass

@main.command()
def clean():
    """Delete UnUsed File"""
    helpers.clean_directory()
    pass


if __name__ == '__main__':
    main()


import setuptools

setuptools.setup(
    name="Grank",
    version="0.0.11",
    author="Bestony@LCTT",
    author_email="xiqingongzi@gmail.com",
    description="A Github Project Rank Command Line Tool",
    long_description=open('README.rst').read(),
    url="https://github.com/LCTT/Grank",
    project_urls={
        "Bug Tracker": "https://github.com/LCTT/Grank/issues",
        "Documentation": "https://github.com/LCTT/Grank",
        "Source Code": "https://github.com/LCTT/Grank",
    },
    license="GPLv3",
    py_modules=['grank'],
    packages=setuptools.find_packages(),
    install_requires= [
        'click',
        'requests',
        'pandas',
        'numpy',
        'matplotlib'
    ],
    entry_points="""
        [console_scripts]
        grank=grank.core:main
    """,
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
)